@@ -3,12 +3,13 @@
 from .devices import OmadaSwitchPortDetails
 from .omadaclient import OmadaClient, OmadaSite
 from .omadasiteclient import (
-    OmadaSiteClient,
-    SwitchPortOverrides,
     AccessPointPortSettings,
     GatewayPortSettings,
     OmadaClientSettings,
     OmadaClientFixedAddress,
+    OmadaSiteClient,
+    PortProfileOverrides,
+    SwitchPortSettings,
 )
 from . import definitions
 from . import exceptions
@@ -22,7 +23,8 @@ __all__ = [
     "GatewayPortSettings",
     "OmadaClientSettings",
     "OmadaClientFixedAddress",
-    "SwitchPortOverrides",
+    "PortProfileOverrides",
+    "SwitchPortSettings",
     "OmadaSwitchPortDetails",
     "definitions",
     "exceptions",
