@@ -2,7 +2,7 @@
 
 from argparse import _SubParsersAction
 from tplink_omada_client.cli.command_client import print_client
-from tplink_omada_client.omadasiteclient import OmadaClientSettings
+from tplink_omada_client import OmadaClientSettings
 from .config import get_target_config, to_omada_connection
 from .util import dump_raw_data, get_client_mac, get_target_argument
 
