@@ -16,6 +16,7 @@ from .definitions import (
     LinkDuplex,
     LinkSpeed,
     LinkStatus,
+    NetworkTagsSetting,
     OmadaApiData,
     PoEMode,
     PortType,
@@ -45,32 +46,32 @@ class OmadaDevice(OmadaApiData):
     @property
     def name(self) -> str:
         """The device name."""
-        return self._data["name"]
+        return self._data.get("name", self.mac)
 
     @property
     def model(self) -> str:
         """The device model, such as EAP225."""
-        return self._data["model"]
+        return self._data("model", "Unknown")
 
     @property
     def model_display_name(self) -> str:
         """Model description for front-end display."""
-        return self._data["showModel"]
+        return self._data.get("showModel", "Unknown Model")
 
     @property
     def status(self) -> DeviceStatus:
         """The status of the device."""
-        return DeviceStatus(self._data["status"])
+        return DeviceStatus(self._data.get("status", DeviceStatus.UNKNOWN))
 
     @property
     def status_category(self) -> DeviceStatusCategory:
         """The high-level status of the device."""
-        return DeviceStatusCategory(self._data["statusCategory"])
+        return DeviceStatusCategory(self._data.get("statusCategory", DeviceStatusCategory.UNKNOWN))
 
     @property
     def ip_address(self) -> str:
         """IP address of the device."""
-        return self._data["ip"]
+        return self._data.get("ip", "")
 
     @property
     def display_uptime(self) -> str | None:
@@ -107,7 +108,10 @@ class OmadaDevice(OmadaApiData):
     @property
     def firmware_version(self) -> str:
         """Firmware version of the device"""
-        return self._data["firmwareVersion"]
+        if self._data["statusCategory"] == DeviceStatusCategory.CONNECTED:
+            return self._data["firmwareVersion"]
+        else:
+            return "Unknown"
 
 
 class OmadaListDevice(OmadaDevice):
@@ -136,7 +140,7 @@ class OmadaDetailedDevice(OmadaDevice):
     @property
     def led_setting(self) -> LedSetting:
         """The onboard LED setting for the device"""
-        return LedSetting(self._data["ledSetting"])
+        return LedSetting(self._data.get("ledSetting", LedSetting.UNKNOWN))
 
 
 class OmadaLink(OmadaApiData):
@@ -294,17 +298,17 @@ class OmadaSwitchDeviceCaps(OmadaApiData):
     @property
     def poe_ports(self) -> int:
         """Number of PoE ports supported."""
-        return self._data["poePortNum"]
+        return self._data.get("poePortNum", 0)
 
     @property
     def supports_poe(self) -> bool:
         """Is PoE supported."""
-        return self._data["poeSupport"]
+        return self._data.get("poeSupport", False)
 
     @property
     def supports_bt(self) -> bool:
         """Is BT supported."""
-        return self._data["supportBt"]
+        return self._data.get("supportBt", False)
 
 
 class OmadaSwitch(OmadaDetailedDevice):
@@ -316,12 +320,12 @@ class OmadaSwitch(OmadaDetailedDevice):
         if "portNum" in self._data:
             return self._data["portNum"]
         # So much for the docs
-        return self._data["deviceMisc"]["portNum"]
+        return self._data.get("deviceMisc", {}).get("portNum", 0)
 
     @property
     def ports(self) -> list[OmadaSwitchPort]:
         """List of ports attached to the switch."""
-        return [OmadaSwitchPort(p) for p in self._data["ports"]]
+        return [OmadaSwitchPort(p) for p in self._data.get("ports", [])]
 
     @property
     def uplink(self) -> OmadaUplink | None:
@@ -343,7 +347,7 @@ class OmadaSwitch(OmadaDetailedDevice):
     @property
     def device_capabilities(self) -> OmadaSwitchDeviceCaps:
         """Capabilities of the switch."""
-        return OmadaSwitchDeviceCaps(self._data["devCap"])
+        return OmadaSwitchDeviceCaps(self._data.get("devCap", {}))
 
 
 class OmadaAccesPointLanPortSettings(OmadaApiData):
@@ -352,12 +356,12 @@ class OmadaAccesPointLanPortSettings(OmadaApiData):
     @property
     def port_name(self) -> str:
         """Name of the port - can't be edited"""
-        return self._data["lanPort"]
+        return self._data.get("lanPort", "LAN Port")
 
     @property
     def supports_vlan(self) -> bool:
         """True if the port supports VLAN tagging"""
-        return self._data["supportVlan"]
+        return self._data.get("supportVlan", False)
 
     @property
     def local_vlan_enable(self) -> bool:
@@ -381,7 +385,7 @@ class OmadaAccesPointLanPortSettings(OmadaApiData):
 
         WARNING: Do not enable PoE for EAPs powered from another EAP
         """
-        return self._data["poeOutEnable"]
+        return self._data.get("poeOutEnable", False)
 
 
 class OmadaAccessPoint(OmadaDetailedDevice):
@@ -390,37 +394,47 @@ class OmadaAccessPoint(OmadaDetailedDevice):
     @property
     def wireless_linked(self) -> bool:
         """True, if the AP is connected wirelessley."""
-        return self._data["wirelessLinked"]
+        return self._data.get("wirelessLinked", False)
 
     @property
     def supports_5g(self) -> bool:
         """True if 5G wifi is supported"""
-        return self._data["deviceMisc"]["support5g"]
+        if "deviceMisc" not in self._data:
+            return False
+        return self._data["deviceMisc"].get("support5g", False)
 
     @property
     def supports_5g2(self) -> bool:
         """True if 5G2 wifi is supported"""
-        return self._data["deviceMisc"]["support5g2"]
+        if "deviceMisc" not in self._data:
+            return False
+        return self._data["deviceMisc"].get("support5g2", False)
 
     @property
     def supports_6g(self) -> bool:
         """True if Wifi 6 is supported"""
-        return self._data["deviceMisc"]["support6g"]
+        if "deviceMisc" not in self._data:
+            return False
+        return self._data["deviceMisc"].get("support6g", False)
 
     @property
     def supports_11ac(self) -> bool:
         """True if PoE is supported"""
-        return self._data["deviceMisc"]["support11ac"]
+        if "deviceMisc" not in self._data:
+            return False
+        return self._data["deviceMisc"].get("support11ac", False)
 
     @property
     def supports_mesh(self) -> bool:
         """True if mesh networking is supported"""
-        return self._data["deviceMisc"]["supportMesh"]
+        if "deviceMisc" not in self._data:
+            return False
+        return self._data["deviceMisc"].get("supportMesh", False)
 
     @property
     def lan_port_settings(self) -> list[OmadaAccesPointLanPortSettings]:
         """Settings for the LAN ports on the access point"""
-        return [OmadaAccesPointLanPortSettings(p) for p in self._data["lanPortSettings"]]
+        return [OmadaAccesPointLanPortSettings(p) for p in self._data.get("lanPortSettings", [])]
 
     @property
     def wired_uplink(self) -> OmadaUplink | None:
@@ -464,10 +478,16 @@ class OmadaSwitchPortDetails(OmadaSwitchPort):
         """True if the port's config profile has been overridden."""
         return self._data["profileOverrideEnable"]
 
+    @property
+    def supports_poe(self) -> bool:
+        """True if the port supports PoE."""
+        return self._data.get("supportPoe", True)  # default to true, some older versions don't report this
+
     @property
     def poe_mode(self) -> PoEMode:
         """PoE config for this port."""
-        return PoEMode(self._data.get("poe", PoEMode.NONE))
+        # For reasons, Omada may claim Enabled on non-poe ports - probably due to port profiles
+        return PoEMode(self._data.get("poe", PoEMode.NONE)) if self.supports_poe else PoEMode.NONE
 
     @property
     def bandwidth_limit_mode(self) -> BandwidthControl:
@@ -501,27 +521,97 @@ class OmadaSwitchPortDetails(OmadaSwitchPort):
     @property
     def lldp_med_enabled(self) -> bool:
         """LLDP Mode"""
-        return self._data["lldpMedEnable"]
+        return self._data.get("lldpMedEnable", False)
 
     @property
     def topology_notify_enabled(self) -> bool:
         """Topology notify mode"""
-        return self._data["topoNotifyEnable"]
+        return self._data.get("topoNotifyEnable", False)
 
     @property
     def spanning_tree_enabled(self) -> bool:
         """Spanning tree loopback control"""
-        return self._data["spanningTreeEnable"]
+        return self._data.get("spanningTreeEnable", False)
 
     @property
     def loopback_detect_enabled(self) -> bool:
         """Loopback detection"""
-        return self._data["loopbackDetectEnable"]
+        return self._data.get("loopbackDetectEnable", False)
 
     @property
     def port_isolation_enabled(self) -> bool:
         """Port isolation (Danger!)"""
-        return self._data["portIsolationEnable"]
+        return self._data.get("portIsolationEnable", False)
+
+    @property
+    def loopback_detect_vlan_based_enabled(self) -> bool:
+        """VLAN based loopback detection"""
+        return self._data.get("loopbackDetectVlanBasedEnable", False)
+
+    @property
+    def has_loopback_detect_vlan_based(self) -> bool:
+        """True if VLAN based loopback detection settings are available for this port"""
+        return "loopbackDetectVlanBasedEnable" in self._data
+
+    @property
+    def native_network_id(self) -> str | None:
+        """ID of the port's native network."""
+        return self._data.get("nativeNetworkId", None)
+
+    @property
+    def tag_ids(self) -> list[str]:
+        """IDs of the port's tagged networks."""
+        return self._data.get("tagIds", [])
+
+    @property
+    def network_tags_setting(self) -> NetworkTagsSetting:
+        """Network tags setting for the port."""
+        return self._data.get("networkTagsSetting", NetworkTagsSetting.UNKNOWN)
+
+    @property
+    def tagged_network_ids(self) -> list[str]:
+        """IDs of the port's tagged networks."""
+        return self._data.get("tagNetworkIds", [])
+
+    @property
+    def untagged_network_ids(self) -> list[str]:
+        """IDs of the port's untagged networks."""
+        return self._data.get("untagNetworkIds", [])
+
+    @property
+    def voice_network_enabled(self) -> bool:
+        """True if Voice Network is enabled for the port"""
+        return self._data.get("voiceNetworkEnable", False)
+
+    @property
+    def has_voice_network(self) -> bool:
+        """True if Voice Network settings are available for this port"""
+        return "voiceNetworkEnable" in self._data
+
+    @property
+    def voice_network_id(self) -> str | None:
+        """ID of the port's voice network, if any."""
+        return self._data.get("voiceNetworkId", None)
+
+    @property
+    def flow_control_enabled(self) -> bool:
+        """Flow control enabled"""
+        return self._data.get("flowControlEnable", False)
+
+    @property
+    def has_flow_control(self) -> bool:
+        """True if Flow Control settings are available for this port"""
+        return "flowControlEnable" in self._data
+
+    @property
+    def eee_enabled(self) -> bool:
+        """Energy Efficient Ethernet enabled"""
+        return self._data.get("eeeEnable", False)
+
+    @property
+    def has_eee(self) -> bool:
+        """True if EEE settings are available for this port"""
+        return "eeeEnable" in self._data
 
 
 class OmadaPortProfile(OmadaApiData):
@@ -560,27 +650,57 @@ class OmadaPortProfile(OmadaApiData):
     @property
     def lldp_med_enabled(self) -> bool:
         """LLDP Mode"""
-        return self._data["lldpMedEnable"]
+        return self._data.get("lldpMedEnable", False)
 
     @property
     def topology_notify_enabled(self) -> bool:
         """Topology notify mode"""
-        return self._data["topoNotifyEnable"]
+        return self._data.get("topoNotifyEnable", False)
 
     @property
     def spanning_tree_enabled(self) -> bool:
         """Spanning tree loopback control"""
-        return self._data["spanningTreeEnable"]
+        return self._data.get("spanningTreeEnable", False)
 
     @property
     def loopback_detect_enabled(self) -> bool:
         """Loopback detection"""
-        return self._data["loopbackDetectEnable"]
+        return self._data.get("loopbackDetectEnable", False)
 
     @property
     def port_isolation_enabled(self) -> bool:
         """Port isolation (Danger!)"""
-        return self._data["portIsolationEnable"]
+        return self._data.get("portIsolationEnable", False)
+
+    @property
+    def loopback_detect_vlan_based_enabled(self) -> bool:
+        """VLAN based loopback detection"""
+        return self._data.get("loopbackDetectVlanBasedEnable", False)
+
+    @property
+    def has_loopback_detect_vlan_based(self) -> bool:
+        """True if VLAN based loopback detection settings are available for this profile"""
+        return "loopbackDetectVlanBasedEnable" in self._data
+
+    @property
+    def flow_control_enabled(self) -> bool:
+        """Flow control enabled"""
+        return self._data.get("flowControlEnable", False)
+
+    @property
+    def has_flow_control(self) -> bool:
+        """True if Flow Control settings are available for this profile"""
+        return "flowControlEnable" in self._data
+
+    @property
+    def eee_enabled(self) -> bool:
+        """Energy Efficient Ethernet enabled"""
+        return self._data.get("eeeEnable", False)
+
+    @property
+    def has_eee(self) -> bool:
+        """True if EEE settings are available for this profile"""
+        return "eeeEnable" in self._data
 
 
 class OmadaInterfaceDetails(OmadaApiData):
@@ -781,17 +901,17 @@ class OmadaGateway(OmadaDetailedDevice):
     @property
     def supports_poe(self) -> bool:
         """True if the device supports PoE."""
-        return self._data["supportPoe"]
+        return self._data.get("supportPoe", False)
 
     @property
     def ip(self) -> str:
         """Gateway's LAN IP address."""
-        return self._data["ip"]
+        return self._data.get("ip", "")
 
     @property
     def port_status(self) -> list[OmadaGatewayPortStatus]:
         """Status of the gateway's ports."""
-        return [OmadaGatewayPortStatus(p) for p in self._data["portStats"]]
+        return [OmadaGatewayPortStatus(p) for p in self._data.get("portStats", [])]
 
     @property
     def port_configs(self) -> list[OmadaGatewayPortConfig]:
