@@ -27,7 +27,7 @@ class OmadaApiConnection:
 
     _own_session: bool
     _controller_id: str
-    _controller_version: str
+    _controller_version: str | None = None
     _csrf_token: str | None
     _last_logon: float
 
@@ -92,7 +92,7 @@ class OmadaApiConnection:
         version, controller_id = await self._get_controller_info()
 
         if AwesomeVersion(version) < AwesomeVersion("5.1.0"):
-            raise UnsupportedControllerVersion(self._controller_version)
+            raise UnsupportedControllerVersion(version)
 
         self._controller_id = controller_id
         self._controller_version = version
@@ -137,6 +137,14 @@ class OmadaApiConnection:
 
         return urljoin(self._url, f"/{self._controller_id}/api/v2/{end_point}")
 
+    def format_openapi_url(self, end_point: str, site: str | None = None) -> str:
+        """Get a REST url for the controller action"""
+
+        if site:
+            end_point = f"/sites/{site}/{end_point}"
+
+        return urljoin(self._url, f"/openapi/v1/{self._controller_id}{end_point}")
+
     async def iterate_pages(self, url: str, params: dict[str, Any] | None = None) -> AsyncIterable[dict[str, Any]]:
         """Iterates all the entries of a paged endpoint"""
         request_params = {}
@@ -169,6 +177,13 @@ class OmadaApiConnection:
 
         return await self._do_request(method, url, params=params, json=json, data=data)
 
+    async def get_controller_version(self) -> AwesomeVersion:
+        """Get the controller version as an AwesomeVersion object."""
+        version = self._controller_version
+        if version is None:
+            version, _ = await self._get_controller_info()
+        return AwesomeVersion(version)
+
     async def _do_request(self, method: str, url: str, params=None, json=None, data: Payload | None = None) -> Any:
         """Perform a request on the controller, and unpack the response."""
 
@@ -179,6 +194,9 @@ class OmadaApiConnection:
         headers = {}
         if self._csrf_token:
             headers["Csrf-Token"] = self._csrf_token
+            headers["Omada-Request-Source"] = "web-local"
+            headers["Referer"] = self._url + "/"
+            headers["Origin"] = self._url
 
         try:
             async with session.request(
